/***************************************************************************
 *   Copyright (C) 2005 by Pawel Nawrocki                                  *
 *   pnawrocki@interia.pl                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "wlassistant.h"
#include "netlistviewitem.h"
#include "waconfig.h"
#include "ui_netparamswizard.h"
#include "ui_netparamsedit.h"
#include <iostream>

#include <qregexp.h>
#include <qlabel.h>
#include <qprocess.h>
#include <qcursor.h>
#include <qeventloop.h>
#include <qtimer.h>
#include <qcursor.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qwidgetstack.h>
#include <qtooltip.h>

#include <kpushbutton.h>
#include <kcombobox.h>
#include <klistview.h>
#include <kapplication.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <klocale.h>

WirelessAssistant::WirelessAssistant(QWidget* parent, const char* name, bool modal, WFlags fl)
                : mainWindow(parent,name, modal,fl)
{
        buttonScan->setIconSet( SmallIconSet("find") );
        buttonConnect->setIconSet( SmallIconSet("connect_creating") );
        buttonOptions->setIconSet( SmallIconSet("configure") );
        buttonClose->setIconSet( SmallIconSet("fileclose") );

        netList->setAllColumnsShowFocus(1);
        netList->setItemMargin(8);
        frameDevice->hide();

        /// Network List Widget
        connect( buttonScan, SIGNAL(clicked()),
                 this, SLOT(netScan()) );

        connect( buttonConnect, SIGNAL(clicked()),
                 this, SLOT(itemAction()) );

        connect( buttonClose, SIGNAL(clicked()),
                 this, SLOT(close()) );

        connect( devCombo, SIGNAL(activated( const QString & )),
                 this, SLOT(setDev( const QString & )) );

        connect( netList, SIGNAL(rightButtonPressed( QListViewItem*, const QPoint&, int )),
                 SLOT(showItemContextMenu( QListViewItem*, const QPoint&, int )) );

        /// Settings Widget
        connect( buttonOptions, SIGNAL(toggled(bool)),
                 this, SLOT(togglePage(bool)) );

        connect( buttonEnableAllMessages, SIGNAL(clicked()),
                 this, SLOT(enableAllMessages()) );

        /// Global KDE Options
        connect( KApplication::kApplication(), SIGNAL(settingsChanged(int)),
                 this, SLOT(updateConfiguration(int)) );

        setMouseBehaviour();

        QTimer::singleShot(10, this, SLOT(init()) ); //WAIT FOR THE UI TO BE READY BEFORE FURTHER SETUP (msec)
}

WirelessAssistant::~WirelessAssistant()
{}

/*$SPECIALIZATION$*/

void WirelessAssistant::init()
{
        ////////////////////////////////////////
        ///// CHECK FOR SYSFS (KERNEL 2.6) /////
        if ( !QFile::exists("/sys") ) {
                std::cout << "Sysfs not present." << std::endl;
                KMessageBox::error( 0, i18n("Kernel 2.6 or later not present.\nWireless Assistant will now quit.") );
                close();
                return;
        }

        /////////////////////////////////////////////////////
        ///// LOAD CONFIG FILE INCL. ALL NET PARAMETERS /////
        WaSettings::self()->addItemBool("Auto Quit", autoQuit);
        WaSettings::self()->addItemInt("DHCP Client Timeout", DhcpTimeout);
        WaSettings::self()->addItemString("Interface", NetParams.iface);
        WaSettings::self()->addItemStringList("NetParamsList", NetParamsList );
        WaSettings::self()->readConfig();
        checkAutoQuit->setChecked(autoQuit);
        if (!DhcpTimeout)
                DhcpTimeout = spinDhcpTimeout->value();
        else
                spinDhcpTimeout->setValue(DhcpTimeout);

        ///////////////////////////////////
        ///// DETECT WIRELESS DEVICES /////
        statusLabel->setText(i18n("Detecting wireless devices..."));
        QString devPath = "/sys/class/net/";
        QDir *devDir = new QDir( devPath );
        devDir->setFilter( QDir::Dirs | QDir::Readable );
        QStringList devList = devDir->entryList();
        delete devDir;

        std::cout << "Wireless interface(s): ";
        for ( QStringList::Iterator dev = devList.begin(); dev != devList.end(); dev++ ) {
                if ( *dev == "." || *dev == ".." || *dev == "lo" || (*dev).left(3) == "sit" )
                        continue;
                if ( ((*dev).left(4) == "wlan" || (*dev).left(3) == "ath") || QDir( QString(devPath+(*dev)+"/wireless") ).exists() ) {
                        std::cout << (*dev);
                        devCombo->insertItem(*dev);
                }
        }

        statusLabel->setText(i18n("Finished detecting devices."));
        if (devCombo->count() == 0) {
                std::cout << "none" << std::endl;
                KMessageBox::error(0, i18n("No usable wireless device found.\nWireless Assistant will now quit."));
                close();
                return;
        }
        std::cout << std::endl;
        if (devCombo->count() > 1) {
                for (int i=0; i<devCombo->count(); i++) {
                        if ( devCombo->text(i)==NetParams.iface ) {	//if the item matches interface saved in config file - select it.
                                devCombo->setCurrentItem( i );
                                break;
                        }
                }
                frameDevice->show();
        }
        NetParams.iface = devCombo->currentText();	// set interface name

        //////////////////////////////////
        ///// CHECK FILE PERMISSIONS /////
        if (!QFileInfo("/etc/resolv.conf").isWritable()) {
                std::cout << "warning: /etc/resolv.conf not writable" << std::endl;
                KMessageBox::information(0, i18n("<qt><p>You might have insufficient permissions for Wireless Assistant to function properly.</p><p>Did you run it using '<tt>sudo</tt>'?</p></qt>") );
        }

        ///////////////////////////////////////
        ///// INITIALIZE GLOBAL VARIABLES /////
        Commands.init();
        connectedItem = 0;
        timerGui = new QTimer();
        timerConnectionCheck = new QTimer();
        connect( timerGui, SIGNAL(timeout()), SLOT(updateConnectedItem()) );
        connect( timerConnectionCheck, SIGNAL(timeout()), SLOT(checkConnectionStatus()) );

        ////////////////////////
        ///// DETECT PATHS /////
        if (!Commands.allFound) {	//all ok or ONLY dhcpcd not found (i.e. dhclient present).
                KMessageBox::error(0, i18n("Executable(s) '%1' could not be found.\nWireless Assistant will now quit.").arg(Commands.notFound.join("', '")) );
                close();
                return;
        }

        ///////////////////////////////////////
        ///// SCAN FOR AVAILABLE NETWORKS /////
        netScan( NetParams );
}

void WirelessAssistant::checkConnectionStatus()
{
        if ( isConnected( true ) )
                return;
        else {
                netList->setSelected( connectedItem, true );
                setNetParamsFromList( connectedItem );
                setConnectedItemByAp( 0 );
                if ( KMessageBox::questionYesNo(0, i18n("Connection to '%1' has been lost!\nWould you like to reconnect?").arg(NetParams.essid), i18n("Connection Lost") , KStdGuiItem::yes(), KStdGuiItem::no(), "AutoReconnect" ) == KMessageBox::Yes ) {
                        netDisconnect( true );
                        netConnect();
                } else
                        return;
        }
}

QString WirelessAssistant::getConnectedAp( const WANetParams & np )
{
        QString a;
        QString s = runCommand( Commands.cmd("iwconfig_status", np, true ) );
        a = getVal(s, "Access Point\\W+(\\S+)" );
        return a;
}

int WirelessAssistant::getConnectedQuality( const WANetParams & np )
{
        int q1, q2, q;
        QString s = runCommand( Commands.cmd("iwconfig_status", np, true ) );
        q1 = getVal(s, "Link Quality\\W+(\\d+)").toInt();
        q2 = getVal(s, "Link Quality\\W+\\d+/(\\d+)").toInt();
        q = (94*q1/q2);

        return q;
}

void WirelessAssistant::removeNetParams()
{
        NetListViewItem *nvi = static_cast<NetListViewItem*>(netList->selectedItem());
        QString ap = nvi->ap();
        for (QStringList::Iterator nps = NetParamsList.begin(); nps != NetParamsList.end(); nps++) {
                if ( (*nps).section(",",2,2)==ap ) {
                        if ( KMessageBox::warningContinueCancel(0, i18n("<qt><p>Settings for network '<b>%1</b>' are about to be deleted.</p><p>Would you like to continue?</p></qt>").arg(matchEssidForAp(ap))) == KMessageBox::Continue ) {
                                if (nvi->hidden())	// hiddenEssid = 1
                                        nvi->setEssid("<hidden>");
                                NetParamsList.remove(nps);
                                WaSettings::writeConfig();
                                statusLabel->setText( i18n("Settings deleted.") );
                        }
                        break;
                }
        }
}


void WirelessAssistant::setDNS( const WANetParams & np )
{
        QFile f("/etc/resolv.conf");
        if (f.open( IO_WriteOnly | IO_Truncate )) {
                QTextStream s( &f );
                if (!np.domain.isEmpty()) {
                        s << QString("domain " + np.domain + "\n");
                        std::cout << "resolv.conf: domain " << np.domain << std::endl;
                }
                if (!np.dns1.isEmpty()) {
                        s << QString("nameserver " + np.dns1 + "\n");
                        std::cout << "resolv.conf: nameserver " << np.dns1 << std::endl;
                }
                if (!np.dns2.isEmpty()) {
                        s << QString("nameserver " + np.dns2 + "\n");
                        std::cout << "resolv.conf: nameserver " << np.dns2 << std::endl;
                }
                f.close();
        } else {
                std::cout << "dns setup error: " << f.name() << " is not writeable." << std::endl;
                KMessageBox::error(0, i18n("<qt><p>File '<i>%1</i>' could not be opened for writing.</p><p>Nameserver(s) and/or domain are not set.</p></qt>").arg(f.name()) );
        }
}

void WirelessAssistant::netScan()
{
        netScan( NetParams );
}

void WirelessAssistant::netScan( const WANetParams & np )
{
        if (!radioEnabled()) {
                statusLabel->setText("Radio off. Scanning aborted.");
                setUi(1);
                return;
        }

        setUi(0);

        bool wasConnected = false;
        if (connectedItem)
                wasConnected = true;
        else if (isConnected( true ))
                wasConnected = true;

        statusLabel->setText(i18n("Bringing interface %1 up...").arg(np.iface));
        runCommand( Commands.cmd("ifup",np) );

        setConnectedItemByAp( 0 );
        netList->clear();

        QString result;
        statusLabel->setText(i18n("Scanning..."));
        result = runCommand( Commands.cmd("scan",np) );

        parseScan( result );

        if (netList->childCount() > 0) {
                std::cout << "Networks found: " << QString::number( netList->childCount() ) << std::endl;

                if (wasConnected) {
                        setConnectedItemByAp( getConnectedAp( NetParams ));
                } else {
                        runCommand( Commands.cmd("ifdown", NetParams) ); // Workaround for drivers that always use cached results (madwifi...) and report same scan results/error even when changed/fixed. Bringing interface down seems to improve the situation.
                }
                statusLabel->setText( i18n("Done.") );
        } else {
                //Workaround like above.
                runCommand( Commands.cmd("ifdown", NetParams) );
                std::cout << "No networks found!" << std::endl;
                statusLabel->setText( i18n("No networks found.") );

                if ( result.find("Resource temporarily unavailable")>-1 ) {
                        std::cout << "Radio switch is off." << std::endl;
                        KMessageBox::information(0, i18n("Radio of your wireless card is turned off using an external switch on your computer.\nYou need turn it on to be able to use wireless networks.") );
                }
        }

        netList->triggerUpdate();
        setNetListColumns();
}

void WirelessAssistant::parseScan( const QString & output )
{
        QString essid;
        QString channel;
        QString mode;
        int qualInt;
        bool enc;
        bool hidden;
        QString ap;

        bool ok_channel = true;	//does iwlist return channel?
        QString section;

        netList->setUpdatesEnabled( false ); //do not redraw while adding items to avoid flicker.

        for (int i=1; (!output.section("Cell ",i,i).isEmpty()); i++ ) {
                section = output.section("Cell ",i,i);

                // GET ESSID VALUE
                essid = getVal(section, "ESSID\\W+\"(.+)\"");

                // GET CHANNEL NUMBER
                channel = getVal(section, "Channel\\D+(\\d+)" );
                if (channel.isEmpty()) {
                        channel = getVal(section, "Frequency\\D+(\\d.+)Hz");
                        ok_channel = false;
                }

                // GET MODE VALUE
                /*mode = getVal(output, "Mode:"," ","Cell ",i);

                if (mode.upper()=="AD-HOC")
                        continue; // Ad-Hoc connections not supported.

                if (mode.upper()=="MASTER")
                        mode="Managed"; //Some drivers (madwifi...) report Managed networks as Master. Workaround.*/

                // GET AP
                ap = getVal(section, "Address\\W+(\\S+)");

                if (essid.isEmpty()) {
                        if (!ap.isEmpty()) //older wireless-tools report "", not "<hidden>"
                                essid = "<hidden>";
                        else
                                continue;	//some cards report one '' essid even when no network's present. Workaround.
                }

                if (essid=="<hidden>") {
                        hidden = true;
                        essid = matchEssidForAp( ap );
                } else
                        hidden=false;

                // GET QUALITY
                int wsignal;
                //check if driver reports quality directly
                qualInt = getVal(section, "Quality\\D+(\\d+)").toInt();

                if (qualInt == 0) { //noise not reported? estimate.
                        wsignal = getVal(section, "Signal level\\D+(\\d+)" ).toInt();
                        qualInt = 100-wsignal;
                        if (qualInt <1)
                                qualInt=1;
                }

                if (qualInt>94)
                        qualInt=94;


                // GET ENCRYPTION
                if (getVal(section, "Encryption key\\W+(\\w+)" ).upper()=="OFF")
                        enc = false;
                else
                        enc = true;

                new NetListViewItem( netList, essid, channel, qualInt, enc, ap, hidden );
        }

        if (!ok_channel)
                netList->setColumnText( 1, i18n("Freq (Hz)") );

        netList->setUpdatesEnabled( true );
        setUi(1);

        //QTimer::singleShot(10, this, SLOT(setNetListColumns()) ); /// HACK: using QTimer, otherwise quality column reports wrong width. netList is already set, so WHY DOES THIS HAPPEN?
}

bool WirelessAssistant::radioEnabled()
{
        bool r;
        if ( runCommand( Commands.cmd("iwconfig_status", NetParams, true ) ).find( QRegExp("Tx-Power.off" ))>-1) {
                if (KMessageBox::questionYesNo(0, "Radio of your wireless card is off.\nWould you like to turn it on?")== KMessageBox::Yes) {
                        runCommand( Commands.cmd("radio_on", NetParams) );
                        r = true;
                } else {
                        r = false;
                }
        } else
                r = true;

        return r;
}

void WirelessAssistant::setNetParamsFromList( QListViewItem* lvi )
{
        NetListViewItem *nvi = static_cast<NetListViewItem*>(lvi);
        NetParams.essid = nvi->essid();
        NetParams.hiddenEssid = nvi->hidden();
        //NetParams.mode = nvi->mode();
        NetParams.channel = nvi->channel();
        NetParams.ap = nvi->ap();
        NetParams.wep = nvi->enc();
}

bool WirelessAssistant::setNetParamsFromConfig( const QString & ap )
{
        for (QStringList::Iterator nps = NetParamsList.begin(); nps != NetParamsList.end(); nps++) {
                if ( (*nps).section(",",2,2)==ap ) {
                        NetParams.loadNetParamsString( *nps );
                        return 1;
                }
        }
        return 0;
}

void WirelessAssistant::itemAction()
{
        QListViewItem* lvi = netList->selectedItem();
        if (!lvi)
                return;

        NetListViewItem* nvi = static_cast<NetListViewItem*>(lvi);
        ///////////////////
        ///// ACTIONS /////
        if (nvi->isConnected()) {
                std::cout << "ACTION: DISCONNECT." << std::endl;
                netDisconnect();
                return;
        } else {
                std::cout << "ACTION: CONNECT." << std::endl;
                netConnect();
                return;
        }
}

void WirelessAssistant::netConnect()
{
        setNetParamsFromList( netList->selectedItem() );

        if ( (NetParams.essid=="<hidden>") || (!setNetParamsFromConfig( NetParams.ap )) ) {
                ui_NetParamsWizard *netwiz = new ui_NetParamsWizard;
                if (!NetParams.hiddenEssid)
                        netwiz->setCaption( i18n("%1 - First Connection Wizard").arg(NetParams.essid) );
                netwiz->setEssidEnabled( NetParams.hiddenEssid );
                netwiz->setWepEnabled( NetParams.wep );
                netwiz->exec();
                if (netwiz->result()==QDialog::Rejected) {
                        delete netwiz;
                        return;
                } else {
                        NetParams = netwiz->readNetParams( NetParams );
                        NetParams.wasHiddenEssid = NetParams.hiddenEssid; //first time values.
                        NetParams.wasWep = NetParams.wep;
                        NetParamsList << NetParams.netParamsString();
                        if (NetParams.hiddenEssid)
                                static_cast<NetListViewItem*>(netList->selectedItem())->setEssid( NetParams.essid );
                        WaSettings::self()->writeConfig();
                        delete netwiz;
                }
        }

        if (NetParams.review())
                editNetParams();
        updateNetParams();
        netConnect( NetParams );
}

void WirelessAssistant::updateNetParams()
{
        for (QStringList::Iterator nps = NetParamsList.begin(); nps != NetParamsList.end(); nps++) {
                if ( (*nps).section(",",2,2)==NetParams.ap ) {
                        QString newNps = NetParams.netParamsString();
                        if ( newNps!=(*nps) ) {
                                (*nps) = newNps;
                                WaSettings::self()->writeConfig();
                                std::cout << "Network settings updated." << std::endl;
                                statusLabel->setText( i18n("Network settings updated.") );
                                break;
                        }
                }
        }
}

QString WirelessAssistant::matchEssidForAp( const QString & ap )
{
        for (QStringList::Iterator nps = NetParamsList.begin(); nps != NetParamsList.end(); nps++) {
                if ( (*nps).section(",",2,2)==ap ) {
                        return (*nps).section(",",1,1); //essid
                }
        }
        return "<hidden>";
}

void WirelessAssistant::netConnect( const WANetParams & np )
{
        setUi(0);

        if (connectedItem)
                netDisconnect( true );

        statusLabel->setText( i18n("Connecting to '%1'...").arg(np.essid) );
        statusLabel->repaint();
        runCommand( Commands.cmd("ifup", np) );
        runCommand( Commands.cmd("iwconfig_set", np) );
        runCommand( Commands.cmd("iwconfig_ap", np) );
        if (np.dhcp) { //DHCP config
                QString dhcp_out = runCommand( Commands.cmd("ifconfig_dhcp", np), DhcpTimeout );
                if (dhcp_out.find("ERR::killed")>-1) {
                        setUi(1);
                        std::cout << "CONNECTION FAILED (DHCP client interrupted/timed out)." << std::endl;
                        statusLabel->setText("Connection failed.");
                        return;
                }
        } else { //manual config
                runCommand( Commands.cmd("ifconfig_manual", np) );
                setDNS( np );
                runCommand( Commands.cmd("route", np) );
        }
        //////////////////////
        ///// CHECK CONNECTION
        statusLabel->setText(i18n("Testing connection..."));
        statusLabel->repaint();

        if (isConnected()) {
                if (autoQuit)
                        this->close();
                setConnectedItemByAp( np.ap );
                statusLabel->setText( i18n("Successfully connected to '%1'.").arg(np.essid) );
                setUi(1);
        } else {
                statusLabel->setText(i18n("Connection failed."));
                setConnectedItemByAp( 0 );
                std::cout << "Kill stale DHCP client (if exists)." << std::endl;
                runCommand( Commands.cmd("kill_dhcp", np) ); //kill any stale DHCP client running (seems it's dhclient only)
                setUi(1);
                if (KMessageBox::questionYesNo(0, i18n("Connection failed.\nWould you like to review settings for this network?"), i18n("Review Settings?") , KStdGuiItem::yes(), KStdGuiItem::no(), "ReviewSettings" ) == KMessageBox::Yes)
                        editNetParams();
        }
}

void WirelessAssistant::updateConnectedItem()
{
        int q = getConnectedQuality( NetParams );
        connectedItem->setQuality( q );
}

void WirelessAssistant::setConnectedItemByAp( const QString & ap )
{
        if (connectedItem) {
                timerGui->stop();
                timerConnectionCheck->stop();
                connectedItem->setConnected( false );
                connectedItem = 0;
        }
        if (!ap.isEmpty()) {
                QListViewItem* lvi = getItemByAp( ap );
                if (lvi) {
                        if (getConnectedQuality( NetParams )==0)
                                return;	// don't set as connected if link quality is 0
                        NetListViewItem* nvi = static_cast<NetListViewItem*>(lvi);
                        nvi->setConnected( true );
                        connectedItem = nvi;
                        netList->sort();	// sort to make sure new connectedItem is 1st.
                }
        }

        if (connectedItem) {
                timerGui->start(2500); //update quality indicator every 2.5seconds
                timerConnectionCheck->start(10000); //check if connection still active every 10seconds
        }
        updateConnectButton( netList->selectedItem() );
}

void WirelessAssistant::netDisconnect( const bool & quiet )
{
        if ( (quiet) || (KMessageBox::warningContinueCancel(0, i18n("<qt><p>You are about to disconnect from '<b>%1</b>'.</p><p>Would you like to continue?<p></qt>").arg(connectedItem->essid()) )== KMessageBox::Continue ) ) {
                statusLabel->setText( i18n("Disconnecting...") );
                statusLabel->repaint();
                setConnectedItemByAp( 0 );
                if ( runCommand( Commands.cmd( "kill_dhcp", NetParams ) ).isEmpty() ) { //no output means all ok.
                        statusLabel->setText( i18n("Waiting for DHCP client to shut down...") );
                        statusLabel->repaint();
                        QTimer* tmr = new QTimer();
                        tmr->start(1500, true);	//wait 1.5sec for dhcp client to really shutdown, single shot.
                        while ( tmr->isActive() ) {
                                KApplication::eventLoop()->processEvents( QEventLoop::AllEvents );
                                usleep(75*1000); //75msec on Linux
                        }
                        delete tmr;
                }
                runCommand( Commands.cmd( "disconnect", NetParams ) );
                runCommand( Commands.cmd( "ifdown", NetParams ) );

                std::cout << "DISCONNECTED." << std::endl;
                statusLabel->setText( i18n("Done.") );
        } else {
                statusLabel->setText( i18n("Cancelled.") );
        }
}

QListViewItem* WirelessAssistant::getItemByAp( const QString & ap )
{
        QListViewItem* lvi = netList->firstChild();
        while (lvi) {
                if ( static_cast<NetListViewItem*>(lvi)->
                                ap() == ap ) {
                        break;
                }
                lvi = lvi->nextSibling();
        }
        return lvi;
}

void WirelessAssistant::updateConfiguration(int category)
{
        if (category == KApplication::SETTINGS_MOUSE) {
                setMouseBehaviour();
                return;
        }
        if (category == -1) {
                autoQuit = checkAutoQuit->isChecked();
                DhcpTimeout = spinDhcpTimeout->value();
        }
}

void WirelessAssistant::togglePage(bool options)
{
        buttonScan->setDisabled(options);
        buttonConnect->setDisabled(options);
        if (options) {
                if (WaSettings::self()->config()->groupList().contains("Notification Messages")>0)
                        buttonEnableAllMessages->setEnabled(true);
                else
                        buttonEnableAllMessages->setEnabled(false);
                widgetStack->raiseWidget(optionsPage);
        } else {
                widgetStack->raiseWidget(netPage);
                updateConfiguration(-1);
        }
}

void WirelessAssistant::enableAllMessages()
{
        KMessageBox::enableAllMessages();
        buttonEnableAllMessages->setEnabled( false );
}

void WirelessAssistant::setMouseBehaviour()
{
        if ( KGlobalSettings::singleClick() ) {
                disconnect( netList, SIGNAL(selectionChanged(QListViewItem*)),
                            this, SLOT(updateConnectButton(QListViewItem*)) );
                disconnect( netList, SIGNAL(doubleClicked(QListViewItem*, const QPoint &, int)),
                            this, SLOT(itemAction()) );
                connect( netList, SIGNAL(clicked(QListViewItem*, const QPoint &, int)),
                         this, SLOT(itemAction()) );
                buttonConnect->hide();
        } else {
                disconnect( netList, SIGNAL(clicked(QListViewItem*, const QPoint &, int)),
                            this, SLOT(itemAction()) );

                connect( netList, SIGNAL(selectionChanged(QListViewItem*)),
                         this, SLOT(updateConnectButton(QListViewItem*)) );
                connect( netList, SIGNAL(doubleClicked(QListViewItem*, const QPoint &, int)),
                         this, SLOT(itemAction()) );
                buttonConnect->show();
        }
}

void WirelessAssistant::updateConnectButton(QListViewItem* lvi)
{
        QToolTip::remove
                (buttonConnect);
        if ( lvi == connectedItem ) {
                buttonConnect->setText( i18n("&Disconnect") );
                QToolTip::add
                        ( buttonConnect, i18n("Disconnect from the selected network") );

        } else {
                buttonConnect->setText( i18n("&Connect") );
                QToolTip::add
                        ( buttonConnect, i18n("Connect from the selected network") );

        }
}

void WirelessAssistant::setDev( const QString & ifname)
{
        NetParams.iface = ifname;
        std::cout << "Selected interface: " << ifname << std::endl;
        netScan();
}

/*QString WirelessAssistant::getIfName(QString ifName)
{
        QString devId,devName,execpath;
        int marker,len;
 
        // get device id
        execpath=getPath("udevinfo");
        if (execpath.isEmpty())
                return QString::null;
        QProcess* proc = new QProcess( this );
        proc->addArgument(execpath);
        proc->addArgument("-ap");
        proc->addArgument( QString("/class/net/"+ifName) );
        if (!proc->start())
                return QString::null;
while (proc->isRunning()) {}
        devId = QString(proc->readStdout());
        marker=devId.find("\"device\"");
        marker=devId.find("ID=",marker);
        marker=devId.find(":",marker)+1;
        len = devId.find("\"",marker)-marker;
        devId=devId.mid(marker, len);
 
        //get human-readable interface name
        execpath=getPath("lspci");
        if (execpath.isEmpty())
                return QString::null;
        proc = new QProcess( this );
        proc->addArgument(execpath);
        if (!proc->start())
                return QString::null;
while (proc->isRunning()) {}
        devName = QString(proc->readStdout());
        marker=devName.find(devId)+devId.length();
        marker=devName.find(":",marker)+2;
        len=devName.find("(",marker)-1-marker;
        devName=devName.mid(marker,len);
 
        delete proc;
        return devName;
}*/

QString WirelessAssistant::runCommand( const QStringList & cmd, int timeout )
{
        QTimer* timerProc = new QTimer(); //timeout timer
        if (cmd.isEmpty())
                return QString::null;

        QProcess* p = new QProcess( this );
        p->setArguments( cmd );

        if (timeout>0) {
                connect( timerProc, SIGNAL(timeout()), p, SLOT(kill()) );
                timerProc->start(timeout*1000); //convert sec to msec
        }

        int i = 0;
        p->start();
        while ( p->isRunning() ) { // PROCESS USER EVENTS
                KApplication::eventLoop()->processEvents( QEventLoop::AllEvents );
                usleep(75*1000); //75msec on Linux (75000msec on Windows...)
                if (i==20) { //1.5sec have passed and the process is still running. Replace the 'Close' button with 'Stop'.
                        disconnect(buttonClose, SIGNAL(clicked()),
                                   this, SLOT(close()) );
                        connect(buttonClose, SIGNAL(clicked()),
                                p, SLOT(kill()) );
                        buttonClose->setIconSet( SmallIconSet("stop") );
                        buttonClose->setText( i18n("&Stop") );
                        QToolTip::remove
                                (buttonClose);
                        QToolTip::add
                                ( buttonClose, i18n("Terminate current process") );
                }
                i++;
        }

        if (i>20) {//set 'stop' back to 'close' if needed
                disconnect(buttonClose, SIGNAL(clicked()),
                           p, SLOT(kill()) );
                connect(buttonClose, SIGNAL(clicked()),
                        this, SLOT(close()) );
                buttonClose->setIconSet( SmallIconSet("fileclose") );
                buttonClose->setText( i18n("&Quit") );
                QToolTip::remove
                        (buttonClose);
                QToolTip::add
                        ( buttonClose, i18n("Quit the application") );
        }

        if (timerProc->isActive())
                timerProc->stop();
        delete timerProc;
        QString e = QString( p->readStderr() );
        QString o = QString( p->readStdout() );
        if (!p->normalExit()) {
                o.append("::ERR::killed");
                std::cout << "Process terminated (timed out)." << std::endl;
        }
        delete p;
        if (!e.isEmpty()) {
                std::cout << "==>stderr: " << e;// << std::endl;
                o.append("::ERR::");
                o.append(e);
        }

        return o;
}

void WirelessAssistant::setUi(int uiState)
{

        if (uiState==0) {
                devCombo->setEnabled( false );
                buttonScan->setEnabled( false );
                buttonConnect->setEnabled( false );
                buttonOptions->setEnabled( false );
                KApplication::setOverrideCursor( QCursor(Qt::BusyCursor) );
        } else {
                if (devCombo->count() > 0) {
                        devCombo->setEnabled( true );
                        buttonScan->setEnabled( true );
                }
                if (netList->childCount() > 0)
                        buttonConnect->setEnabled( true );
                buttonOptions->setEnabled( true );
                KApplication::restoreOverrideCursor();
        }
}

void WirelessAssistant::showItemContextMenu( QListViewItem* i, const QPoint& p, int c )
{
        if (!i)
                return;

        NetListViewItem *nvi = static_cast<NetListViewItem*>(i);
        QString ap = nvi->ap();
        bool isConfigured = setNetParamsFromConfig(ap);
        KPopupMenu *icm = new KPopupMenu();
        icm->insertTitle(nvi->essid());
        if (isConfigured) {
                if (nvi->isConnected()) {
                        icm->insertItem( SmallIcon("connect_no"), i18n("Disconnect..."), this, SLOT(netDisconnect()) );
                        //icm->insertItem( SmallIcon("reload"), i18n("Reconnect"), this, SLOT(netConnect()) );
                } else
                        icm->insertItem( SmallIcon("connect_creating"), i18n("Connect"), this, SLOT(netConnect()) );
                icm->insertSeparator();
                icm->insertItem(i18n("Forget Settings..."), this, SLOT(removeNetParams()) );
                icm->insertItem(i18n("Edit Settings..."), this, SLOT(editNetParams()) );
        } else {
                if (nvi->isConnected()) {
                        icm->insertItem( SmallIcon("connect_no"), i18n("Disconnect..."), this, SLOT(netDisconnect()) );
                        //icm->insertItem( SmallIcon("reload"), i18n("Configure and Reconnect..."), this, SLOT(netConnect()) );
                } else
                        icm->insertItem( SmallIcon("connect_creating"), i18n("Configure and Connect..."), this, SLOT(netConnect()) );
        }
        icm->exec( QCursor::pos() );
}

void WirelessAssistant::editNetParams()
{
        setNetParamsFromList( netList->selectedItem() );	//prepate NetParams
        setNetParamsFromConfig( NetParams.ap );	//prepate NetParams

        ui_NetParamsEdit *netedit = new ui_NetParamsEdit();
        netedit->setValues( NetParams );
        netedit->setCaption( i18n("%1 Settings").arg(NetParams.essid) );
        netedit->exec();
        if (netedit->result() == QDialog::Rejected) {
                delete netedit;
                return;
        } else {	//accepted
                NetParams = netedit->readNetParams( NetParams );
                updateNetParams();
        }
}

void WirelessAssistant::setNetListColumns()
{
        int realWidth = netList->viewportSize( netList->contentsWidth(), netList->contentsHeight() ).width(); //calculate actual width taking scrollbars into account
        int essidWidth = realWidth - netList->columnWidth(1) - netList->columnWidth(2) - netList->columnWidth(3);

        netList->setColumnWidth(0, essidWidth);
        netList->triggerUpdate();
}

bool WirelessAssistant::isConnected( const bool & quiet )
{
        QString result;
        if (quiet) {
                std::cout << "Checking connection." << std::endl;
                result = runCommand( Commands.cmd("ping_internet", NetParams, true ), 3 ); //no errors from ping command.
                if ( (result.find("::ERR::")>-1) || (result.find("unknown host")>-1) || (result.find("unreachable")>-1) || (result.find("100%")>-1)  )
                        return false;
                else
                        return true;
        } else {
                if (!NetParams.dhcp) {
                        result = runCommand( Commands.cmd( "ping_gw", NetParams ) );
                        if ( (result.find("::ERR::")>-1) || (result.find("unknown host")>-1) || (result.find("unreachable")>-1) || (result.find("100%")>-1)  ) {
                                std::cout << "==>error: could not ping gateway (" << NetParams.gateway << ")\n";
                                std::cout << "CONNECTION FAILED" << std::endl;
                                return 0;
                        } else
                                std::cout << "Gateway successfully reached." << std::endl;
                }
                result = runCommand( Commands.cmd( "ping_internet", NetParams ) );
                if ( (result.find("::ERR::")>-1) || (result.find("unknown host")>-1) || (result.find("unreachable")>-1) || (result.find("100%")>-1)  ) {
                        std::cout << "==>error: could not ping internet\n";
                        std::cout << "CONNECTION FAILED" << std::endl;
                        return 0;
                } else {
                        std::cout << "Internet successfully reached\nCONNECTION OK" << std::endl;
                        return 1;
                }
        }
}

QString WirelessAssistant::getVal(const QString & str, const QString & rxs)
{
        QRegExp rx(rxs);
        rx.search(str);
        return rx.cap(1).stripWhiteSpace();
}

bool WirelessAssistant::close()
{
        updateConfiguration(-1);	//read values from setingsPage;
        WaSettings::writeConfig();
        std::cout << "Application options saved." << std::endl;
        return QWidget::close();
}


#include "wlassistant.moc"
